'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _WEBGL_LIMITS;

exports.getContextLimits = getContextLimits;
exports.getParameter = getParameter;

var _constants = require('./constants');

var _constants2 = _interopRequireDefault(_constants);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; } // A major polyfill of "gl.getParameter"
// Attempts to return sane values for all known WebGL2 and extension constants
/* eslint-disable camelcase */


// Return true if WebGL2 context
function isWebGL2(gl) {
  var GL_TEXTURE_BINDING_3D = 0x806A;
  return gl && gl.TEXTURE_BINDING_3D === GL_TEXTURE_BINDING_3D;
}

var WEBGL_debug_renderer_info = 'WEBGL_debug_renderer_info';
var EXT_disjoint_timer_query = 'EXT_disjoint_timer_query';
var EXT_disjoint_timer_query_webgl2 = 'EXT_disjoint_timer_query_webgl2';
var EXT_texture_filter_anisotropic = 'EXT_texture_filter_anisotropic';

var WEBGL_LIMITS = (_WEBGL_LIMITS = {}, _defineProperty(_WEBGL_LIMITS, _constants2.default.ALIASED_LINE_WIDTH_RANGE, [new Float32Array([1, 1])]), _defineProperty(_WEBGL_LIMITS, _constants2.default.ALIASED_POINT_SIZE_RANGE, [new Float32Array([1, 1])]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_TEXTURE_SIZE, [64, 2048]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_CUBE_MAP_TEXTURE_SIZE, [16]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_TEXTURE_IMAGE_UNITS, [8]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_COMBINED_TEXTURE_IMAGE_UNITS, [8]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VERTEX_TEXTURE_IMAGE_UNITS, [0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_RENDERBUFFER_SIZE, [1]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VARYING_VECTORS, [8]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VERTEX_ATTRIBS, [8]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VERTEX_UNIFORM_VECTORS, [128]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_FRAGMENT_UNIFORM_VECTORS, [16]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VIEWPORT_DIMS, [new Int32Array([0, 0])]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_3D_TEXTURE_SIZE, [0, 256]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_ARRAY_TEXTURE_LAYERS, [0, 256]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_CLIENT_WAIT_TIMEOUT_WEBGL, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_COLOR_ATTACHMENTS, [0, 4]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_COMBINED_UNIFORM_BLOCKS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_DRAW_BUFFERS, [0, 4]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_ELEMENT_INDEX, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_ELEMENTS_INDICES, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_ELEMENTS_VERTICES, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_FRAGMENT_INPUT_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_FRAGMENT_UNIFORM_BLOCKS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_FRAGMENT_UNIFORM_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_SAMPLES, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_SERVER_WAIT_TIMEOUT, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_TEXTURE_LOD_BIAS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_UNIFORM_BLOCK_SIZE, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_UNIFORM_BUFFER_BINDINGS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VARYING_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VERTEX_OUTPUT_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VERTEX_UNIFORM_BLOCKS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_VERTEX_UNIFORM_COMPONENTS, [0, 0]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MIN_PROGRAM_TEXEL_OFFSET, [0, -8]), _defineProperty(_WEBGL_LIMITS, _constants2.default.MAX_PROGRAM_TEXEL_OFFSET, [0, 7]), _defineProperty(_WEBGL_LIMITS, _constants2.default.UNIFORM_BUFFER_OFFSET_ALIGNMENT, [0, 0]), _WEBGL_LIMITS);

// const EXTENSION_SETTINGS = {
//   [GL.MAX_TEXTURE_MAX_ANISOTROPY_EXT]: 'anisotropy'
// };

function getContextLimits(gl) {
  gl.luma = gl.luma || {};

  if (!gl.luma.limits) {
    gl.luma.limits = {};
    gl.luma.webgl1MinLimits = {};
    gl.luma.webgl2MinLimits = {};

    var isWebgl2 = isWebGL2(gl);

    // WEBGL limits
    for (var parameter in WEBGL_LIMITS) {
      var limit = WEBGL_LIMITS[parameter];

      var webgl1MinLimit = limit.gl1;
      var webgl2MinLimit = 'gl2' in limit ? limit.gl2 : limit.gl1;
      var minLimit = isWebgl2 ? webgl2MinLimit : webgl1MinLimit;

      // Check if we can query for this limit
      var limitNotAvailable = 'gl2' in limit && !isWebgl2 || 'extension' in limit && !gl.getExtension(limit.extension);

      var value = limitNotAvailable ? minLimit : gl.getParameter(parameter);
      gl.luma.limits[parameter] = value;
      gl.luma.webgl1MinLimits[parameter] = webgl1MinLimit;
      gl.luma.webgl2MinLimits[parameter] = webgl2MinLimit;
    }
  }

  return gl.luma.limits;
}

function getParameter(gl, originalFunc, pname) {
  var GL_UNMASKED_VENDOR_WEBGL = 0x9245; // vendor string of the graphics driver.
  var GL_UNMASKED_RENDERER_WEBGL = 0x9246; // renderer string of the graphics driver.

  var GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT = 0x84FF;

  var GL_FRAGMENT_SHADER_DERIVATIVE_HINT = 0x8B8B;
  var GL_DONT_CARE = 0x1100;
  var GL_GPU_DISJOINT_EXT = 0x8FBB;

  var extensions = gl.luma.extensions;


  var info = gl.getExtension(WEBGL_debug_renderer_info);

  switch (pname) {
    // EXTENSIONS SOMETIMES DO NOT USE THE OFFICIAL CONSTANTS.
    case GL_UNMASKED_VENDOR_WEBGL:
      return originalFunc(info && info.UNMASKED_VENDOR_WEBGL || _constants2.default.VENDOR);
    case GL_UNMASKED_RENDERER_WEBGL:
      return originalFunc(info && info.UNMASKED_RENDERER_WEBGL || _constants2.default.RENDERER);

    case GL_FRAGMENT_SHADER_DERIVATIVE_HINT:
      return !isWebGL2(gl) ? GL_DONT_CARE : undefined;

    case GL_GPU_DISJOINT_EXT:
      var hasTimerQueries = !extensions[EXT_disjoint_timer_query] && !extensions[EXT_disjoint_timer_query_webgl2];
      return hasTimerQueries ? 0 : undefined;

    case GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT:
      var ext = gl.luma.extensions[EXT_texture_filter_anisotropic];
      pname = ext && ext.MAX_TEXTURE_MAX_ANISOTROPY_EXT;
      return !pname ? 1.0 : undefined;

    default:
      return undefined;
  }
}
//# sourceMappingURL=get-parameter.js.map